@@ -1,35 +1,43 @@
-#' @title Estimate optimal gamma
+#' @title Estimate gamma
 #' @name gamma_GMM
 #' 
 #' @description
 #' This function minimizes the Euclidean distance between the theoretical
-#' skewness of a skewed Lambert W x Gaussian random variable and the 
-#' sample skewness of the back-transformed
-#' data \eqn{W_{\delta}(\boldsymbol z)} as a function of \eqn{\gamma} (see
-#' References). Only an interative application of this function will
-#' give a good estimate of \eqn{\gamma} (see \code{\link{IGMM}}).
+#'     skewness of a skewed Lambert W x Gaussian random variable and the sample
+#'     skewness of the back-transformed data \eqn{W_{\gamma}(\boldsymbol z)} as
+#'     a function of \eqn{\gamma} (see References). Only an interative
+#'     application of this function will give a good estimate of \eqn{\gamma}
+#'     (see \code{\link{IGMM}}).
 #' 
 #' @param z a numeric vector of data values.
-#' @param gamma.init starting value for \eqn{\gamma}; default: \code{\link{gamma_Taylor}}.
-#' @param skewness.x theoretical skewness of the input \eqn{X}; default: \code{0}.
+#' 
+#' @param gamma.init starting value for \eqn{\gamma}; default:
+#'     \code{\link{gamma_Taylor}}.
+#' 
+#' @param skewness.x theoretical skewness of the input \eqn{X}; default:
+#'     \code{0}.
+#'
 #' @param robust logical; if \code{TRUE}, robust measure of asymmetry
-#' (\code{\link{mc}}) will be used; default: \code{FALSE}.
-#' @param tol a positive scalar giving the tolerance at which the distance is
-#' considered close enough to zero to terminate the algorithm; default:
-#' \code{.Machine$double.eps^0.25}.
+#'     (\code{\link{medcouple_estimator}}) will be used; default: \code{FALSE}.
+#'
+#' @param tol a positive scalar; tolerance level for terminating the iterative
+#'     algorithm; default: \code{.Machine$double.eps^0.25}.
+#'
 #' @param not.negative logical; if \code{TRUE}, the estimate for \eqn{\gamma} is
-#' restricted to non-negative reals, which be useful for scale-family
-#' Lambert W\eqn{\times} F random variables. Default: \code{FALSE}.
-#' @param optim.fct string; which R optimization function should be used.
-#' By default it uses \code{\link[stats]{optimize}} which is about 
-#' 8-10x faster than \code{\link[stats]{nlminb}}.
+#'     restricted to non-negative reals, which is useful for scale-family
+#'     Lambert W\eqn{\times} F random variables. Default: \code{FALSE}.
+#'
+#' @param optim.fct string; which R optimization function should be used.  By
+#'     default it uses \code{\link[stats]{optimize}} which is about 8-10x faster
+#'     than \code{\link[stats]{nlminb}}.
+#'
 #' @return 
 #' A list with two elements: 
 #' \item{gamma}{ scalar; optimal \eqn{\gamma}, } 
 #' \item{iterations}{number of iterations (\code{NA} for \code{"optimize"}).}
 #' @seealso 
 #' \code{\link{delta_GMM}} for the heavy-tail version of this
-#' function; \code{\link{mc}} for a robust measure of asymmetry;
+#' function; \code{\link{medcouple_estimator}} for a robust measure of asymmetry;
 #' \code{\link{IGMM}} for an iterative method to estimate all parameters
 #' jointly.
 #' @keywords optimize
@@ -42,7 +50,8 @@
 #' gamma_GMM(y, optim.fct = "nlminb")
 #' gamma_GMM(y)
 #' 
-gamma_GMM <- function(z, skewness.x = 0, gamma.init = gamma_Taylor(z), robust = FALSE, 
+gamma_GMM <- function(z, skewness.x = 0, gamma.init = gamma_Taylor(z),
+                      robust = FALSE, 
                       tol = .Machine$double.eps^0.25, not.negative = FALSE,
                       optim.fct = c("optimize", "nlminb")) {
   
@@ -60,7 +69,7 @@ gamma_GMM <- function(z, skewness.x = 0, gamma.init = gamma_Taylor(z), robust =
       if (!robust) {
         empirical.skewness <- skewness(u.d) 
       } else {
-        empirical.skewness <- mc(u.d)
+        empirical.skewness <- medcouple_estimator(u.d)
       }
       return(lp_norm(empirical.skewness - skewness.x, 2))
     }
@@ -77,7 +86,8 @@ gamma_GMM <- function(z, skewness.x = 0, gamma.init = gamma_Taylor(z), robust =
   }
     
   if (optim.fct == "nlminb") {
-    fit <- nlminb(gamma.init, .obj_fct, lower = lb, upper = ub, control = list(abs.tol = tol))
+    fit <- nlminb(gamma.init, .obj_fct, lower = lb, upper = ub,
+                  control = list(abs.tol = tol))
     out <- list(gamma = fit$par,
                 iterations = fit$iterations)
   } else if (optim.fct == "optimize") {
